@@ -63,7 +63,8 @@ typedef enum position_type {
     second_axes,
     graph,
     screen,
-    character
+    character,
+    polar_axes
 } position_type;
 
 /* A full 3D position, with all 3 coordinates of possible using different axes.
@@ -82,7 +83,7 @@ typedef struct text_label {
     enum JUSTIFY pos;		/* left/center/right horizontal justification */
     int rotate;
     int layer;
-    int boxed;			/* EAM_BOXED_TEXT */
+    int boxed;			/* 0 no box;  -1 default box props;  >0 boxstyle */
     char *text;
     char *font;			/* Entry font added by DJL */
     struct t_colorspec textcolor;
@@ -94,11 +95,11 @@ typedef struct text_label {
 
 /* This is the default state for the axis, timestamp, and plot title labels
  * indicated by tag = -2 */
-#define NONROTATABLE_LABEL_TAG -2
+#define NONROTATING_LABEL_TAG -2
 #define ROTATE_IN_3D_LABEL_TAG -3
 #define VARIABLE_ROTATE_LABEL_TAG -4
 #define EMPTY_LABELSTRUCT \
-    {NULL, NONROTATABLE_LABEL_TAG, \
+    {NULL, NONROTATING_LABEL_TAG, \
      {character, character, character, 0.0, 0.0, 0.0}, CENTRE, 0, 0, \
      0, \
      NULL, NULL, {TC_LT, -2, 0.0}, DEFAULT_LP_STYLE_TYPE, \
@@ -109,7 +110,8 @@ typedef struct text_label {
 typedef enum arrow_type {
     arrow_end_absolute,
     arrow_end_relative,
-    arrow_end_oriented
+    arrow_end_oriented,
+    arrow_end_undefined
     } arrow_type;
 
 typedef struct arrow_def {
@@ -122,7 +124,6 @@ typedef struct arrow_def {
     struct arrow_style_type arrow_properties;
 } arrow_def;
 
-#ifdef EAM_OBJECTS
 /* The object types supported so far are OBJ_RECTANGLE, OBJ_CIRCLE, and OBJ_ELLIPSE */
 typedef struct rectangle {
     int type;			/* 0 = corners;  1 = center + size */
@@ -169,8 +170,8 @@ typedef struct object {
     struct object *next;
     int tag;
     int layer;			/* behind or back or front */
-    int object_type;		/* OBJ_RECTANGLE */
-    t_clip_object clip;         
+    int object_type;		/* e.g. OBJ_RECTANGLE */
+    t_clip_object clip;
     fill_style_type fillstyle;
     lp_style_type lp_properties;
     union o {t_rectangle rectangle; t_circle circle; t_ellipse ellipse; t_polygon polygon;} o;
@@ -179,7 +180,6 @@ typedef struct object {
 #define OBJ_CIRCLE (2)
 #define OBJ_ELLIPSE (3)
 #define OBJ_POLYGON (4)
-#endif
 
 /* Datastructure implementing 'set dashtype' */
 struct custom_dashtype_def {
@@ -203,13 +203,34 @@ struct arrowstyle_def {
     struct arrow_style_type arrow_properties;
 };
 
-/* For 'set style parallelaxis' */
+/* Datastructure for 'set pixmap */
+typedef struct t_pixmap {
+    int tag;			/* index referring to this pixmap */
+    struct t_pixmap *next;	/* pointer to next pixmap in the linked list */
+    unsigned int ncols, nrows;	/* image size */
+    t_position pin;		/* where it goes */
+    t_position extent;		/* width dx;  dy implicitly dx*aspect_ratio */
+    int layer;			/* front/back/behind */
+    TBOOLEAN center;		/* position is center rather than lower left */
+    char *filename;		/* where to read the pixmap pixmap */
+    coordval *image_data;	/* pixel array RGBARGBA... */
+} t_pixmap;
+
+/* Used by 'set style parallelaxis' and 'set style spiderplot' */
 struct pa_style {
     lp_style_type lp_properties;/* used to draw the axes themselves */
     int layer;			/* front/back */
 };
 #define DEFAULT_PARALLEL_AXIS_STYLE \
-	{{0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 2.0, 0.0, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, LAYER_FRONT }
+	{{0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 0, 2.0, 0.0, DEFAULT_P_CHAR, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, LAYER_FRONT}
+
+struct spider_web {
+    lp_style_type lp_properties;
+    fill_style_type fillstyle;
+};
+extern struct spider_web spiderplot_style;
+#define DEFAULT_SPIDERPLOT_STYLE \
+	{ DEFAULT_LP_STYLE_TYPE, {FS_EMPTY, 100, 0, DEFAULT_COLORSPEC} }
 
 /* The stacking direction of the key box: (vertical, horizontal) */
 typedef enum en_key_stack_direction {
@@ -244,18 +265,25 @@ typedef enum en_key_sample_positioning {
 } t_key_sample_positioning;
 
 typedef struct {
-    int opt_given; /* option given / not given (otherwise default) */
-    int closeto;   /* from list FILLEDCURVES_CLOSED, ... */
+    enum filledcurves_opts_id closeto;	/* from list FILLEDCURVES_CLOSED, ... */
     double at;	   /* value for FILLEDCURVES_AT... */
     double aty;	   /* the other value for FILLEDCURVES_ATXY */
     int oneside;   /* -1 if fill below bound only; +1 if fill above bound only */
 } filledcurves_opts;
-#define EMPTY_FILLEDCURVES_OPTS { 0, 0, 0.0, 0.0, 0 }
+#define EMPTY_FILLEDCURVES_OPTS { FILLEDCURVES_DEFAULT, 0.0, 0.0, 0 }
 
+typedef enum histogram_type {
+	HT_NONE,
+	HT_STACKED_IN_LAYERS,
+	HT_STACKED_IN_TOWERS,
+	HT_CLUSTERED,
+	HT_ERRORBARS
+} t_histogram_type;
 typedef struct histogram_style {
-    int type;		/* enum t_histogram_type */
+    t_histogram_type type;		/* enum t_histogram_type */
     int gap;		/* set style hist gap <n> (space between clusters) */
     int clustersize;	/* number of datasets in this histogram */
+    TBOOLEAN keyentry;	/* FALSE suppresses extra blank line in key */
     double start;	/* X-coord of first histogram entry */
     double end;		/* X-coord of last histogram entry */
     int startcolor;	/* LT_UNDEFINED or explicit color for first entry */
@@ -264,14 +292,7 @@ typedef struct histogram_style {
     struct histogram_style *next;
     struct text_label title;
 } histogram_style;
-typedef enum histogram_type {
-	HT_NONE,
-	HT_STACKED_IN_LAYERS,
-	HT_STACKED_IN_TOWERS,
-	HT_CLUSTERED,
-	HT_ERRORBARS
-} t_histogram_type;
-#define DEFAULT_HISTOGRAM_STYLE { HT_CLUSTERED, 2, 1, 0.0, 0.0, LT_UNDEFINED, LT_UNDEFINED, 0, NULL, EMPTY_LABELSTRUCT }
+#define DEFAULT_HISTOGRAM_STYLE { HT_CLUSTERED, 2, 1, TRUE, 0.0, 0.0, LT_UNDEFINED, LT_UNDEFINED, 0, NULL, EMPTY_LABELSTRUCT }
 
 typedef enum en_boxplot_factor_labels {
 	BOXPLOT_FACTOR_LABELS_OFF,
@@ -288,22 +309,24 @@ typedef struct boxplot_style {
     TBOOLEAN outliers;
     int pointtype;
     int plotstyle;	/* CANDLESTICKS or FINANCEBARS */
+    double median_linewidth;
     double separation;	/* of boxplots if there are more than one factors */
     t_boxplot_factor_labels labels;	/* Which axis to put the tic labels if there are factors */
     TBOOLEAN sort_factors;	/* Sort factors in alphabetical order? */
 } boxplot_style;
 extern boxplot_style boxplot_opts;
-#define DEFAULT_BOXPLOT_STYLE { 0, 1.5, TRUE, 6, CANDLESTICKS, 1.0, BOXPLOT_FACTOR_LABELS_AUTO, FALSE }
+#define DEFAULT_BOXPLOT_STYLE { 0, 1.5, TRUE, 6, CANDLESTICKS, -1.0, 1.0, BOXPLOT_FACTOR_LABELS_AUTO, FALSE }
 
-#ifdef EAM_BOXED_TEXT
 typedef struct textbox_style {
     TBOOLEAN opaque;	/* True if the box is background-filled before writing into it */
     TBOOLEAN noborder;	/* True if you want fill only, no lines */
     double xmargin;	/* fraction of default margin to use */
     double ymargin;	/* fraction of default margin to use */
+    double linewidth;	/* applied to border */
+    t_colorspec border_color;	/* TC_LT + LT_NODRAW is "noborder" */
+    t_colorspec fillcolor;	/* only used if opaque is TRUE */
 } textbox_style;
-#define DEFAULT_TEXTBOX_STYLE { FALSE, FALSE, 1.0, 1.0 }
-#endif
+#define DEFAULT_TEXTBOX_STYLE { FALSE, FALSE, 1.0, 1.0, 1.0, BLACK_COLORSPEC, BACKGROUND_COLORSPEC }
 
 /***********************************************************/
 /* Variables defined by gadgets.c needed by other modules. */
@@ -329,8 +352,11 @@ typedef struct {
     t_key_region region;	/* if so: where? */
     t_key_ext_region margin;	/* if exterior: where outside? */
     struct position user_pos;	/* if user specified position, this is it */
+    struct position user_width;	/*    "require width <position>" */
+    int user_cols;		/*    "require columns <int>" */
     VERT_JUSTIFY vpos;		/* otherwise these guide auto-positioning */
     JUSTIFY hpos;
+    TBOOLEAN fixed;		/* prevents key in 3D plot from rotating/scaling with plot */
     t_key_sample_positioning just;
     t_key_stack_direction stack_dir;
     double swidth;		/* 'width' of the linestyle sample line in the key */
@@ -345,31 +371,38 @@ typedef struct {
     struct lp_style_type box;	/* linetype of box around key:  */
     char *font;			/* Will be used for both key title and plot titles */
     struct t_colorspec textcolor;	/* Will be used for both key title and plot titles */
+    struct t_colorspec fillcolor;	/* only used if "set key front" */
     BoundingBox bounds;
     int maxcols;		/* maximum no of columns for horizontal keys */
     int maxrows;		/* maximum no of rows for vertical keys */
     text_label title;		/* holds title line for the key as a whole */
+    struct position offset;	/* manual displacement of the entire key */
 } legend_key;
 
 extern legend_key keyT;
 
-#define DEFAULT_KEYBOX_LP {0, LT_NODRAW, 0, DASHTYPE_SOLID, 0, 1.0, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}
+#define DEFAULT_KEYBOX_LP {0, LT_NODRAW, 0, DASHTYPE_SOLID, 0, 0, 1.0, PTSZ_DEFAULT, DEFAULT_P_CHAR, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}
 
 #define DEFAULT_KEY_POSITION { graph, graph, graph, 0.9, 0.9, 0. }
+#define DEFAULT_KEY_WIDTH { graph, graph, graph, 0., 0., 0. }
 
 #define DEFAULT_KEY_PROPS \
 		{ TRUE, \
 		GPKEY_AUTO_INTERIOR_LRTBC, GPKEY_RMARGIN, \
 		DEFAULT_KEY_POSITION, \
-		JUST_TOP, RIGHT, \
+		DEFAULT_KEY_WIDTH, 0, \
+		JUST_TOP, RIGHT, TRUE, \
 		GPKEY_RIGHT, GPKEY_VERTICAL, \
 		4.0, 1.0, 0.0, 0.0, \
 		FILENAME_KEYTITLES, \
 		FALSE, FALSE, FALSE, TRUE, \
 		DEFAULT_KEYBOX_LP, \
 		NULL, {TC_LT, LT_BLACK, 0.0}, \
+		BACKGROUND_COLORSPEC, \
 		{0,0,0,0}, 0, 0, \
-		EMPTY_LABELSTRUCT}
+		EMPTY_LABELSTRUCT, \
+		{character, character, character, 0, 0, 0}, \
+		}
 
 
 /*
@@ -410,6 +443,7 @@ typedef struct t_image {
 } t_image;
 
 extern BoundingBox plot_bounds;	/* Plot Boundary */
+extern BoundingBox page_bounds;	/* 3D boundary prior to view transformation */
 extern BoundingBox canvas; 	/* Writable area on terminal */
 extern BoundingBox *clip_area;	/* Current clipping box */
 
@@ -421,6 +455,9 @@ extern float yoffset;		/* y origin setting */
 extern float aspect_ratio;	/* 1.0 for square */
 extern int aspect_ratio_3D;	/* 2 for equal scaling of x and y; 3 for z also */
 
+extern double boxwidth;
+extern TBOOLEAN boxwidth_is_absolute;
+
 /* plot border autosizing overrides, in characters (-1: autosize) */
 extern t_position lmargin, bmargin, rmargin, tmargin;
 #define DEFAULT_MARGIN_POSITION {character, character, character, -1, -1, -1}
@@ -437,11 +474,12 @@ extern struct linestyle_def *first_mono_linestyle;
 
 extern struct arrowstyle_def *first_arrowstyle;
 
+extern struct t_pixmap *pixmap_listhead;
+
 extern struct pa_style parallel_axis_style;
 
-#ifdef EAM_OBJECTS
 extern struct object *first_object;
-#endif
+extern struct object grid_wall[];
 
 extern text_label title;
 
@@ -450,10 +488,11 @@ extern text_label timelabel;
 /* asctime() format */
 # define DEFAULT_TIMESTAMP_FORMAT "%a %b %d %H:%M:%S %Y"
 #endif
-extern int timelabel_rotate;
 extern int timelabel_bottom;
 
 extern TBOOLEAN	polar;
+extern TBOOLEAN spiderplot;
+extern TBOOLEAN inverted_raxis;	/* true if R_AXIS.set_min > R_AXIS.set_max */
 
 #define ZERO 1e-8		/* default for 'zero' set option */
 extern double zero;		/* zero threshold, not 0! */
@@ -470,7 +509,6 @@ extern t_colorspec background_fill;
 #define border_west	(draw_border & WEST)
 #define border_south	(draw_border & SOUTH)
 #define border_north	(draw_border & NORTH)
-#define border_complete	((draw_border & 15) == 15)
 extern int draw_border;
 extern int user_border;
 extern int border_layer;
@@ -482,6 +520,7 @@ extern const struct lp_style_type default_border_lp;
 extern TBOOLEAN	clip_lines1;
 extern TBOOLEAN	clip_lines2;
 extern TBOOLEAN	clip_points;
+extern TBOOLEAN	clip_radial;
 
 #define SAMPLES 100		/* default number of samples for a plot */
 extern int samples_1;
@@ -493,6 +532,7 @@ extern enum PLOT_STYLE data_style;
 extern enum PLOT_STYLE func_style;
 
 extern TBOOLEAN parametric;
+extern TBOOLEAN in_parametric;
 
 /* If last plot was a 3d one. */
 extern TBOOLEAN is_3d_plot;
@@ -524,85 +564,137 @@ extern TBOOLEAN volatile_data;
 extern int current_x11_windowid;
 
 /* Plot layer definitions are collected here. */
-/* Someday they might actually be used.       */
 #define LAYER_BEHIND     -1
 #define LAYER_BACK        0
 #define LAYER_FRONT       1
-#define LAYER_FOREGROUND  2	/* not currently used */
+#define LAYER_FOREGROUND  2	/* used only for axis tic labels */
+#define LAYER_FRONTBACK   4	/* used only by grid walls */
+#define LAYER_DEPTHORDER  8	/* for objects to be included in pm3d depth sorting */
+#define LAYER_PLOT	 16	/* currently used only by fig.trm */
 #define LAYER_PLOTLABELS 99
 
 /* Functions exported by gadgets.c */
 
+/* initialization (called once on program entry */
+void init_gadgets(void);
+
 /* moved here from util3d: */
-void draw_clip_line __PROTO((int, int, int, int));
-void draw_clip_polygon __PROTO((int , gpiPoint *));
-void draw_clip_arrow __PROTO((int, int, int, int, int));
-void clip_polygon __PROTO((gpiPoint *, gpiPoint *, int , int *));
-int clip_point __PROTO((unsigned int, unsigned int));
-void clip_put_text __PROTO((unsigned int, unsigned int, char *));
+int draw_clip_line(int, int, int, int);
+void draw_clip_polygon(int , gpiPoint *);
+void draw_clip_arrow(double, double, double, double, t_arrow_head);
+void clip_polygon(gpiPoint *, gpiPoint *, int , int *);
+int clip_point(int, int);
 
 /* moved here from graph3d: */
-void clip_move __PROTO((unsigned int x, unsigned int y));
-void clip_vector __PROTO((unsigned int x, unsigned int y));
+void clip_move(int x, int y);
+void clip_vector(int x, int y);
+
+void draw_polar_clip_line(double, double, double, double);
 
 /* Common routines for setting line or text color from t_colorspec */
-void apply_pm3dcolor __PROTO((struct t_colorspec *tc, const struct termentry *t));
-void reset_textcolor __PROTO((const struct t_colorspec *tc, const struct termentry *t));
+void apply_pm3dcolor(struct t_colorspec *tc);
+void reset_textcolor(const struct t_colorspec *tc);
+
+/* Timestamp code shared by 2D and 3D */
+void do_timelabel(int x, int y);
 
 extern fill_style_type default_fillstyle;
 
-#ifdef EAM_OBJECTS
 /*       Warning: C89 does not like the union initializers     */
 extern struct object default_rectangle;
 #define DEFAULT_RECTANGLE_STYLE { NULL, -1, 0, OBJ_RECTANGLE, OBJ_CLIP,	\
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
 	{.rectangle = {0, {0,0,0,0.,0.,0.}, {0,0,0,0.,0.,0.}, {0,0,0,0.,0.,0.}, {0,0,0,0.,0.,0.}}} }
 
 extern struct object default_circle;
 #define DEFAULT_CIRCLE_STYLE { NULL, -1, 0, OBJ_CIRCLE, OBJ_CLIP, \
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
 	{.circle = {1, {0,0,0,0.,0.,0.}, {graph,0,0,0.02,0.,0.}, 0., 360., TRUE }} }
 
 extern struct object default_ellipse;
 #define DEFAULT_ELLIPSE_STYLE { NULL, -1, 0, OBJ_ELLIPSE, OBJ_CLIP, \
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
 	{.ellipse = {ELLIPSEAXES_XY, {0,0,0,0.,0.,0.}, {graph,graph,0,0.05,0.03,0.}, 0. }} }
 
 #define DEFAULT_POLYGON_STYLE { NULL, -1, 0, OBJ_POLYGON, OBJ_CLIP, \
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
 	{.polygon = {0, NULL} } }
 
-#endif
+#define WALL_Y0_TAG 0
+#define WALL_X0_TAG 1
+#define WALL_Y1_TAG 2
+#define WALL_X1_TAG 3
+#define WALL_Z0_TAG 4
+#define WALL_Y0_CORNERS { {graph, graph, graph, 0, 0, 0}, \
+		{graph, graph, graph, 0, 0, 1}, {graph, graph, graph, 1, 0, 1},	\
+		{graph, graph, graph, 1, 0, 0}, {graph, graph, graph, 0, 0, 0} }
+#define WALL_X0_CORNERS { {graph, graph, graph, 0, 0, 0}, \
+		{graph, graph, graph, 0, 1, 0},	{graph, graph, graph, 0, 1, 1}, \
+		{graph, graph, graph, 0, 0, 1}, {graph, graph, graph, 0, 0, 0} }
+#define WALL_Y1_CORNERS { {graph, graph, graph, 0, 1, 0}, \
+		{graph, graph, graph, 1, 1, 0}, {graph, graph, graph, 1, 1, 1},	\
+		{graph, graph, graph, 0, 1, 1}, {graph, graph, graph, 0, 1, 0} }
+#define WALL_X1_CORNERS { {graph, graph, graph, 1, 0, 0}, \
+		{graph, graph, graph, 1, 0, 1},	{graph, graph, graph, 1, 1, 1}, \
+		{graph, graph, graph, 1, 1, 0}, {graph, graph, graph, 1, 0, 0} }
+#define WALL_Z0_CORNERS { {graph, graph, graph, 0, 0, 0}, \
+		{graph, graph, graph, 1, 0, 0}, {graph, graph, graph, 1, 1, 0}, \
+		{graph, graph, graph, 0, 1, 0},	{graph, graph, graph, 0, 0, 0} }
+#define WALL_Y_COLOR 0xcdb79e
+#define WALL_X_COLOR 0x228b22
+#define WALL_Z_COLOR 0xa0b6cd
+
+#define WALL_Y0 { NULL, WALL_Y0_TAG, LAYER_FRONTBACK, OBJ_POLYGON, OBJ_CLIP, \
+	{FS_TRANSPARENT_SOLID, 50, 0, BLACK_COLORSPEC}, \
+	DEFAULT_LP_STYLE_TYPE, \
+	{.polygon = {5, NULL} } }
+#define WALL_Y1 { NULL, WALL_Y1_TAG, LAYER_FRONTBACK, OBJ_POLYGON, OBJ_CLIP, \
+	{FS_TRANSPARENT_SOLID, 50, 0, BLACK_COLORSPEC}, \
+	DEFAULT_LP_STYLE_TYPE, \
+	{.polygon = {5, NULL} } }
+#define WALL_X0 { NULL, WALL_X0_TAG, LAYER_FRONTBACK, OBJ_POLYGON, OBJ_CLIP, \
+	{FS_TRANSPARENT_SOLID, 50, 0, BLACK_COLORSPEC}, \
+	DEFAULT_LP_STYLE_TYPE, \
+	{.polygon = {5, NULL} } }
+#define WALL_X1 { NULL, WALL_X1_TAG, LAYER_FRONTBACK, OBJ_POLYGON, OBJ_CLIP, \
+	{FS_TRANSPARENT_SOLID, 50, 0, BLACK_COLORSPEC}, \
+	DEFAULT_LP_STYLE_TYPE, \
+	{.polygon = {5, NULL} } }
+#define WALL_Z0 { NULL, WALL_Z0_TAG, LAYER_FRONTBACK, OBJ_POLYGON, OBJ_CLIP, \
+	{FS_TRANSPARENT_SOLID, 50, 0, BLACK_COLORSPEC}, \
+	DEFAULT_LP_STYLE_TYPE, \
+	{.polygon = {5, NULL} } }
 
 /* filledcurves style options set by 'set style [data|func] filledcurves opts' */
 extern filledcurves_opts filledcurves_opts_data;
 extern filledcurves_opts filledcurves_opts_func;
 
 /* Prefer line styles over plain line types */
-#if TRUE || defined(BACKWARDS_COMPATIBLE)
+/* Mostly for backwards compatibility */
 extern TBOOLEAN prefer_line_styles;
-#else
-#define prefer_line_styles FALSE
-#endif
 
 extern histogram_style histogram_opts;
 
-#ifdef EAM_BOXED_TEXT
-extern textbox_style textbox_opts;
-#endif
+/* TODO: linked list rather than fixed size array */
+#define NUM_TEXTBOX_STYLES 4
+extern textbox_style textbox_opts[NUM_TEXTBOX_STYLES];
 
-void default_arrow_style __PROTO((struct arrow_style_type *arrow));
-void apply_head_properties __PROTO((struct arrow_style_type *arrow_properties));
+void default_arrow_style(struct arrow_style_type *arrow);
+void apply_head_properties(struct arrow_style_type *arrow_properties);
 
-void free_labels __PROTO((struct text_label *tl));
+void free_labels(struct text_label *tl);
 
-void get_offsets __PROTO((struct text_label *this_label,
-	struct termentry *t, int *htic, int *vtic));
-void write_label __PROTO((unsigned int x, unsigned int y, struct text_label *label));
-int label_width __PROTO((const char *, int *));
+void get_offsets(struct text_label *this_label, int *htic, int *vtic);
+void write_label(int x, int y, struct text_label *label);
+int label_width(const char *, int *);
+
+TBOOLEAN pm3d_objects(void);
+
+void place_title(int title_x, int title_y);
+double effective_aspect_ratio(void);
 
 #endif /* GNUPLOT_GADGETS_H */
