@@ -47,6 +47,7 @@
 #include "version.h"
 #include "voxelgrid.h"
 #include "encoding.h"
+#include "xdg.h"
 
 #include <signal.h>
 #include <setjmp.h>
@@ -358,6 +359,16 @@ main(int argc_orig, char **argv)
     history_init();
 #endif
 #endif
+
+#if defined(HAVE_LIBREADLINE) && defined(RL_VERSION_MAJOR)
+    /* Starting with readline v8.1 bracketed paste mode is on by default.
+     * This breaks multi-line pasted input to gnuplot because it looks like
+     * one long run-on line.
+     */
+    if (RL_VERSION_MAJOR >= 8)
+	rl_variable_bind ("enable-bracketed-paste", "off");
+#endif
+
 #if defined(HAVE_LIBREADLINE) && !defined(MISSING_RL_TILDE_EXPANSION)
     rl_complete_with_tilde_expansion = 1;
 #endif
@@ -366,6 +377,13 @@ main(int argc_orig, char **argv)
 	if (!argv[i])
 	    continue;
 
+	if (!strcmp(argv[i], "-c")) {
+	    /* The rest of the command line is a scriptfile and its arguments.
+	     * Do not try to interpret them here (ignore -V, --help, etc).
+	     */
+	    break;
+	}
+
 	if (!strcmp(argv[i], "-V") || !strcmp(argv[i], "--version")) {
 	    printf("gnuplot %s patchlevel %s\n",
 		    gnuplot_version, gnuplot_patchlevel);
@@ -537,7 +555,19 @@ main(int argc_orig, char **argv)
 	    expanded_history_filename = gp_strdup(GNUPLOT_HISTORY_FILE);
 	    gp_expand_tilde(&expanded_history_filename);
 #endif
-	    read_history(expanded_history_filename);
+	    if (read_history(expanded_history_filename)) {
+#ifdef USE_XDG_BASEDIR
+		/* use $XDG_STATE_DIR/gnuplot_history only if
+		 * $XDG_STATE_DIR exists or can be created */
+		char *xdgpath = xdg_get_path(kXDGStateHome, "gnuplot_history",
+				FALSE /* no subdir */, TRUE /* create */);
+		if (xdgpath) {
+		    free(expanded_history_filename);
+		    expanded_history_filename = xdgpath;
+		    read_history(expanded_history_filename);
+		}
+#endif /* USE_XDG_BASEDIR */
+	    }
 
 	    /*
 	     * It is safe to ignore the return values of 'atexit()' and
@@ -659,9 +689,9 @@ RECOVER_FROM_ERROR_IN_DASH:
 		    gp_exit(EXIT_FAILURE);
 		}
 		call_argc = GPMIN(9, argc - 1);
-		for (i=0; i<=call_argc; i++) {
+		for (i = 0; i < call_argc; i++) {
 		    /* Need to stash argv[i] somewhere visible to load_file() */
-		    call_args[i] = gp_strdup(argv[i+1]);
+		    call_args[i] = gp_strdup(argv[i + 1]);
 		}
 
 		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), 5);
@@ -784,6 +814,7 @@ init_session()
 	successful_initialization = TRUE;
 
 	load_rcfile(2);		/* ~/.gnuplot */
+	load_rcfile(3);		/* ~/.config/gnuplot/gnuplotrc */
 }
 
 /*
@@ -828,6 +859,13 @@ load_rcfile(int where)
 	strcpy(rcfile, user_homedir);
 	PATH_CONCAT(rcfile, PLOTRC);
 	plotrc = fopen(rcfile, "r");
+    } else if (where == 3) {
+#ifdef USE_XDG_BASEDIR
+	rcfile = xdg_get_path(kXDGConfigHome, "gnuplotrc",
+			TRUE /* use subdir */, FALSE /* not create */);
+	if (rcfile)
+	    plotrc = fopen(rcfile, "r");
+#endif
     }
 
     if (plotrc) {
