@@ -113,15 +113,6 @@ static struct udft_entry plot_func;
 
 int plot3d_num=0;
 
-/* FIXME:
- * Because this is global, it gets clobbered if there is more than
- * one unbounded iteration in the splot command, e.g.
- *	splot for [i=0:*] A index i, for [j=0:*] B index j
- * Moving it into (struct surface_points) would be nice but would require
- * extra bookkeeping to track which plot header it is stored in.
- */
-static int last_iteration_in_first_pass = INT_MAX;
-
 /*
  * sp_alloc() allocates a surface_points structure that can hold 'num_iso_1'
  * iso-curves with 'num_samp_2' samples and 'num_iso_2' iso-curves with
@@ -1597,7 +1588,7 @@ eval_3dplots()
      * after the x/yrange is defined.
      */
     plot_iterator = check_for_iteration();
-    last_iteration_in_first_pass = INT_MAX;
+    warn_if_too_many_unbounded_iterations(plot_iterator);
 
     while (TRUE) {
 
@@ -1677,6 +1668,14 @@ eval_3dplots()
 	    else
 		strcpy(c_dummy_var[1], orig_dummy_v_var);
 
+	    /* Make sure there is at least a minimal plot header */
+	    if (*tp_3d_ptr) {
+		this_plot = *tp_3d_ptr;
+	    } else {
+		this_plot = sp_alloc(0, 0, 0, 0);
+		*tp_3d_ptr = this_plot;
+	    }
+
 	    /* Determine whether this plot component is a
 	     *   function (name_str == NULL)
 	     *   data file (name_str is "filename")
@@ -1764,8 +1763,7 @@ eval_3dplots()
 
 		/* for capture to key */
 		this_plot->token = end_token = c_token - 1;
-		/* FIXME: Is this really needed? */
-		this_plot->iteration = plot_iterator ? plot_iterator->iteration : 0;
+		this_plot->iteration = plot_iterator;
 
 		/* this_plot->token is temporary, for errors in get_3ddata() */
 
@@ -1784,15 +1782,9 @@ eval_3dplots()
 	    case SP_KEYENTRY:
 		c_token++;
 		plot_num++;
-		if (*tp_3d_ptr)
-		    this_plot = *tp_3d_ptr;
-		else {		/* no memory malloc()'d there yet */
-		    /* Allocate enough isosamples and samples */
-		    this_plot = sp_alloc(0, 0, 0, 0);
-		    *tp_3d_ptr = this_plot;
-		}
 		this_plot->plot_type = KEYENTRY;
 		this_plot->plot_style = LABELPOINTS;
+		this_plot->title_is_suppressed = FALSE;
 		this_plot->token = end_token = c_token - 1;
 		break;
 
@@ -1857,11 +1849,6 @@ eval_3dplots()
 
 	    } /* End of switch(this_component) */
 
-	    /* clear current title, if it exists */
-	    if (this_plot->title) {
-		free(this_plot->title);
-		this_plot->title = NULL;
-	    }
 
 	    /* default line and point types */
 	    this_plot->lp_properties.l_type = line_num;
@@ -2135,7 +2122,9 @@ eval_3dplots()
 	    this_plot->title_is_automated = FALSE;
 	    if (!set_title) {
 		this_plot->title_no_enhanced = TRUE; /* filename or function cannot be enhanced */
-		if (key->auto_titles == FILENAME_KEYTITLES) {
+		if (key->auto_titles == COLUMNHEAD_KEYTITLES) {
+		    this_plot->title_is_automated = TRUE;
+		} else if (key->auto_titles == FILENAME_KEYTITLES) {
 		    m_capture(&(this_plot->title), start_token, end_token);
 		    if (crnt_param == 2)
 			xtitle = this_plot->title;
@@ -2275,7 +2264,7 @@ eval_3dplots()
 
 		    /* for second pass */
 		    this_plot->token = c_token;
-		    this_plot->iteration = plot_iterator ? plot_iterator->iteration : 0;
+		    this_plot->iteration = plot_iterator;
 
 		    if (this_plot->num_iso_read == 0)
 			this_plot->plot_type = NODATA;
@@ -2287,6 +2276,7 @@ eval_3dplots()
 			count_3dpoints(this_plot, &ntotal, &ninrange, &nundefined);
 			if (ninrange == 0) {
 			    this_plot->plot_type = NODATA;
+			    flag_iteration_nodata(plot_iterator);
 			    goto SKIPPED_EMPTY_FILE;
 			}
 		    }
@@ -2297,24 +2287,18 @@ eval_3dplots()
 		    if (df_return == DF_EOF)
 			break;
 
-		    /* there might be another surface so allocate
-		     * and prepare another surface structure
-		     * This does no harm if in fact there are
-		     * no more surfaces to read
+		    /* There might be another surface so allocate
+		     * and prepare another surface structure.
+		     * This does no harm if in fact there are no more surfaces to read.
+		     * FIXME: There are a lot more flags and setting in the header,
+		     *        like opt_out_of_foo; do we not have to reset them???
+		     *	      I am not sure recycling old headers makes sense any more.
 		     */
-
-		    if ((this_plot = *tp_3d_ptr) != NULL) {
-			if (this_plot->title) {
-			    free(this_plot->title);
-			    this_plot->title = NULL;
-			}
-		    } else {
-			/* Allocate enough isosamples and samples */
+		    if ((this_plot = *tp_3d_ptr) == NULL)
 			this_plot = *tp_3d_ptr = sp_alloc(0, 0, 0, 0);
-		    }
 
 		    this_plot->plot_type = DATA3D;
-		    this_plot->iteration = plot_iterator ? plot_iterator->iteration : 0;
+		    this_plot->iteration = plot_iterator;
 		    this_plot->plot_style = first_dataset->plot_style;
 		    this_plot->lp_properties = first_dataset->lp_properties;
 		    this_plot->fill_properties = first_dataset->fill_properties;
@@ -2353,13 +2337,13 @@ eval_3dplots()
 		   ||  this_plot->plot_type == NODATA) {
 		tp_3d_ptr = &(this_plot->next_sp);
 		this_plot->token = c_token;	/* store for second pass */
-		this_plot->iteration = plot_iterator ? plot_iterator->iteration : 0;
+		this_plot->iteration = plot_iterator;
 
 	    } else if (this_plot->plot_type == VOXELDATA){
 		/* voxel data in an active vgrid must already be present */
 		tp_3d_ptr = &(this_plot->next_sp);
 		this_plot->token = c_token;	/* store for second pass */
-		this_plot->iteration = plot_iterator ? plot_iterator->iteration : 0;
+		this_plot->iteration = plot_iterator;
 		/* FIXME: I worry that vxrange autoscales xrange and xrange autoscales vxrange */
 		autoscale_one_point((&axis_array[FIRST_X_AXIS]), this_plot->vgrid->vxmin);
 		autoscale_one_point((&axis_array[FIRST_X_AXIS]), this_plot->vgrid->vxmax);
@@ -2382,11 +2366,8 @@ eval_3dplots()
 		eof_during_iteration = TRUE;
 	    else if (forever_iteration(plot_iterator) && (this_plot->plot_type != DATA3D))
 		int_error(NO_CARET, "unbounded iteration in something other than a data plot");
-	    else if (forever_iteration(plot_iterator))
-		last_iteration_in_first_pass = plot_iterator->iteration_current;
 
 	    /* restore original value of sample variables */
-	    /* FIXME: somehow this_plot has changed since we saved sample_var! */
 	    if (name_str && this_plot->sample_var) {
 		this_plot->sample_var->udv_value = original_value_u;
 		this_plot->sample_var2->udv_value = original_value_v;
@@ -2402,10 +2383,21 @@ eval_3dplots()
 		break;
 	}
 
-	/* Iterate-over-plot mechanisms */
+	/* Iterate-over-plot mechanisms
+	 * First handle the special case of a nested unbounded iteration
+	 *    splot for [i=a:b] for [j=c:*] ...
+	 * where eof_during_iteration means the inner loop finished but the
+	 * outer loop continues.
+	 * Then handle the usual cases where eof_during_iteration means we're done.
+	 */
+	if (plot_iterator && eof_during_iteration
+	&&  (forever_iteration(plot_iterator->next) < 0)) {
+	    if (next_iteration(plot_iterator)) {
+		c_token = start_token;
+		continue;
+	    }
+	}
 	if (eof_during_iteration) {
-	    FPRINTF((stderr, "eof during iteration current %d\n", plot_iterator->iteration_current));
-	    FPRINTF((stderr, "    last_iteration_in_first_pass %d\n", last_iteration_in_first_pass));
 	    eof_during_iteration = FALSE;
 	} else if (next_iteration(plot_iterator)) {
 	    c_token = start_token;
@@ -2416,13 +2408,10 @@ eval_3dplots()
 	if (equals(c_token, ",")) {
 	    c_token++;
 	    plot_iterator = check_for_iteration();
-	    if (forever_iteration(plot_iterator))
-		if (last_iteration_in_first_pass != INT_MAX)
-		    int_warn(NO_CARET, "splot does not support multiple unbounded iterations");
+	    warn_if_too_many_unbounded_iterations(plot_iterator);
 	} else
 	    break;
-
-    }				/* while(TRUE), ie first pass */
+    }			/* end of first pass, while (TRUE) */
 
 
     if (parametric && crnt_param != 0)
@@ -2504,10 +2493,6 @@ eval_3dplots()
 	c_token = begin_token;
 	plot_iterator = check_for_iteration();
 
-	/* We kept track of the last productive iteration in the first pass */
-	if (forever_iteration(plot_iterator))
-	    plot_iterator->iteration_end = last_iteration_in_first_pass;
-
 	if (hidden3d) {
 	    u_step = (u_max - u_min) / (iso_samples_1 - 1);
 	    v_step = (v_max - v_min) / (iso_samples_2 - 1);
@@ -2596,17 +2581,27 @@ eval_3dplots()
 		    }
 		    /*}}} */
 		}		/* end of ITS A FUNCTION TO PLOT */
+
 		/* we saved it from first pass */
 		c_token = this_plot->token;
 
-		/* we may have seen this one data file in multiple iterations */
-		i = this_plot->iteration;
-		do {
+		/* We may have seen this one data file in multiple iterations.
+		 * Skip over all the plots it may have generated.
+		 * This is *instead of* executing the iterations so ignore
+		 * the current iterator.
+		 */
+		if (this_plot->plot_type == FUNC3D) {
 		    this_plot = this_plot->next_sp;
-		} while (this_plot
-			&& this_plot->token == c_token
-			&& this_plot->iteration == i
-			);
+		} else {
+		    void *it = this_plot->iteration;
+		    plot_iterator = cleanup_iteration(plot_iterator);
+		    do {
+			this_plot = this_plot->next_sp;
+		    } while (this_plot
+			    && this_plot->token == c_token
+			    && this_plot->iteration == it
+			    );
+		}
 
 	    }			/* !is_definition */
 
@@ -2623,8 +2618,6 @@ eval_3dplots()
 		c_token++;
 		if (crnt_param == 0)
 		    plot_iterator = check_for_iteration();
-		if (forever_iteration(plot_iterator))
-		    plot_iterator->iteration_end = last_iteration_in_first_pass;
 	    } else
 		break;
 
