@@ -51,12 +51,14 @@
 #include "pm3d.h"
 #include "getcolor.h"
 
-static void save_functions__sub __PROTO((FILE *));
-static void save_variables__sub __PROTO((FILE *));
-static void save_tics __PROTO((FILE *, struct axis *));
-static void save_mtics __PROTO((FILE *, struct axis *));
-static void save_zeroaxis __PROTO((FILE *,AXIS_INDEX));
-static void save_set_all __PROTO((FILE *));
+static void save_functions__sub(FILE *);
+static void save_variables__sub(FILE *);
+static void save_key(FILE *);
+static void save_tics(FILE *, struct axis *);
+static void save_mtics(FILE *, struct axis *);
+static void save_zeroaxis(FILE *,AXIS_INDEX);
+static void save_set_all(FILE *);
+static void save_justification(int just, FILE *fp);
 
 const char *coord_msg[] = {"first ", "second ", "graph ", "screen ", "character ", "polar "};
 /*
@@ -136,9 +138,9 @@ save_variables__sub(FILE *fp)
 	if (udv->udv_value.type != NOTDEFINED) {
 	    if ((udv->udv_value.type == ARRAY)
 		&& strncmp(udv->udv_name,"ARGV",4)) {
-		    fprintf(fp,"array %s[%d] = ", udv->udv_name,
-			udv->udv_value.v.value_array[0].v.int_val);
-		    save_array_content(fp, udv->udv_value.v.value_array);
+		fprintf(fp,"array %s[%d] = ", udv->udv_name,
+			(int)(udv->udv_value.v.value_array[0].v.int_val));
+		save_array_content(fp, udv->udv_value.v.value_array);
 	    } else if (strncmp(udv->udv_name,"GPVAL_",6)
 		 && strncmp(udv->udv_name,"GPFUN_",6)
 		 && strncmp(udv->udv_name,"MOUSE_",6)
@@ -196,6 +198,35 @@ save_term(FILE *fp)
 	fputs("#    EOF\n", fp);
 }
 
+/* helper function */
+void
+save_axis_label_or_title(FILE *fp, char *name, char *suffix,
+			struct text_label *label, TBOOLEAN savejust)
+{
+    fprintf(fp, "set %s%s \"%s\" ",
+	    name, suffix, label->text ? conv_text(label->text) : "");
+    fprintf(fp, "\nset %s%s ", name, suffix);
+    save_position(fp, &(label->offset), 3, TRUE);
+    fprintf(fp, " font \"%s\"", label->font ? conv_text(label->font) : "");
+    save_textcolor(fp, &(label->textcolor));
+    if (savejust && (label->pos != CENTRE)) save_justification(label->pos,fp);
+    if (label->tag == ROTATE_IN_3D_LABEL_TAG)
+	fprintf(fp, " rotate parallel");
+    else if (label->rotate == TEXT_VERTICAL)
+	fprintf(fp, " rotate");
+    else if (label->rotate)
+	fprintf(fp, " rotate by %d", label->rotate);
+    else
+	fprintf(fp, " norotate");
+
+    if (label == &title && label->boxed) {
+	fprintf(fp," boxed ");
+	if (label->boxed > 0)
+	    fprintf(fp,"bs %d ",label->boxed);
+    }
+    fprintf(fp, "%s\n", (label->noenhanced) ? " noenhanced" : "");
+}
+
 static void
 save_justification(int just, FILE *fp)
 {
@@ -219,7 +250,6 @@ save_set_all(FILE *fp)
     struct arrow_def *this_arrow;
     struct linestyle_def *this_linestyle;
     struct arrowstyle_def *this_arrowstyle;
-    legend_key *key = &keyT;
     int axis;
 
     /* opinions are split as to whether we save term and outfile
@@ -238,10 +268,12 @@ save_set_all(FILE *fp)
     fprintf(fp, "\
 %sset clip points\n\
 %sset clip one\n\
-%sset clip two\n",
+%sset clip two\n\
+%sset clip radial\n",
 	    (clip_points) ? "" : "un",
 	    (clip_lines1) ? "" : "un",
-	    (clip_lines2) ? "" : "un"
+	    (clip_lines2) ? "" : "un",
+	    (clip_radial) ? "" : "un"
 	    );
 
     save_bars(fp);
@@ -269,16 +301,15 @@ save_set_all(FILE *fp)
     else
 	fprintf(fp, "set boxwidth %g %s\n", boxwidth,
 		(boxwidth_is_absolute) ? "absolute" : "relative");
+    fprintf(fp, "set boxdepth %g\n", boxdepth > 0 ? boxdepth : 0.0);
 
     fprintf(fp, "set style fill ");
     save_fillstyle(fp, &default_fillstyle);
 
-#ifdef EAM_OBJECTS
     /* Default rectangle style */
     fprintf(fp, "set style rectangle %s fc ",
 	    default_rectangle.layer > 0 ? "front" :
 	    default_rectangle.layer < 0 ? "behind" : "back");
-    /* FIXME: broke with removal of use_palette? */
     save_pm3dcolor(fp, &default_rectangle.lp_properties.pm3d_color);
     fprintf(fp, " fillstyle ");
     save_fillstyle(fp, &default_rectangle.fillstyle);
@@ -304,7 +335,6 @@ save_set_all(FILE *fp)
 	    fputs("yy\n", fp);
 	    break;
     }
-#endif
 
     if (dgrid3d) {
       if (dgrid3d_mode == DGRID3D_QNORM) {
@@ -396,89 +426,8 @@ save_set_all(FILE *fp)
     /* Save parallel axis state */
     save_style_parallel(fp);
 
-    fprintf(fp, "set key title \"%s\"", conv_text(key->title.text));
-    if (key->title.font)
-	fprintf(fp, " font \"%s\" ", key->title.font);
-    save_justification(key->title.pos, fp);
-    fputs("\n", fp);
-
-    fputs("set key ", fp);
-    switch (key->region) {
-	case GPKEY_AUTO_INTERIOR_LRTBC:
-	    fputs(key->fixed ? "fixed" : "inside", fp);
-	    break;
-	case GPKEY_AUTO_EXTERIOR_LRTBC:
-	    fputs("outside", fp);
-	    break;
-	case GPKEY_AUTO_EXTERIOR_MARGIN:
-	    switch (key->margin) {
-	    case GPKEY_TMARGIN:
-		fputs("tmargin", fp);
-		break;
-	    case GPKEY_BMARGIN:
-		fputs("bmargin", fp);
-		break;
-	    case GPKEY_LMARGIN:
-		fputs("lmargin", fp);
-		break;
-	    case GPKEY_RMARGIN:
-		fputs("rmargin", fp);
-		break;
-	    }
-	    break;
-	case GPKEY_USER_PLACEMENT:
-	    fputs("at ", fp);
-	    save_position(fp, &key->user_pos, 2, FALSE);
-	    break;
-    }
-    if (!(key->region == GPKEY_AUTO_EXTERIOR_MARGIN
-	      && (key->margin == GPKEY_LMARGIN || key->margin == GPKEY_RMARGIN))) {
-	save_justification(key->hpos, fp);
-    }
-    if (!(key->region == GPKEY_AUTO_EXTERIOR_MARGIN
-	      && (key->margin == GPKEY_TMARGIN || key->margin == GPKEY_BMARGIN))) {
-	switch (key->vpos) {
-	    case JUST_TOP:
-		fputs(" top", fp);
-		break;
-	    case JUST_BOT:
-		fputs(" bottom", fp);
-		break;
-	    case JUST_CENTRE:
-		fputs(" center", fp);
-		break;
-	}
-    }
-    fprintf(fp, " %s %s %sreverse %senhanced %s ",
-		key->stack_dir == GPKEY_VERTICAL ? "vertical" : "horizontal",
-		key->just == GPKEY_LEFT ? "Left" : "Right",
-		key->reverse ? "" : "no",
-		key->enhanced ? "" : "no",
-		key->auto_titles == COLUMNHEAD_KEYTITLES ? "autotitle columnhead"
-		: key->auto_titles == FILENAME_KEYTITLES ? "autotitle"
-		: "noautotitle" );
-    if (key->box.l_type > LT_NODRAW) {
-	fputs("box", fp);
-	save_linetype(fp, &(key->box), FALSE);
-    } else
-	fputs("nobox", fp);
-
-    /* These are for the key entries, not the key title */
-    if (key->font)
-	fprintf(fp, " font \"%s\"", key->font);
-    if (key->textcolor.type != TC_LT || key->textcolor.lt != LT_BLACK)
-	save_textcolor(fp, &key->textcolor);
-
-    /* Put less common options on separate lines */
-    fprintf(fp, "\nset key %sinvert samplen %g spacing %g width %g height %g ",
-		key->invert ? "" : "no",
-		key->swidth, key->vert_factor, key->width_fix, key->height_fix);
-    fprintf(fp, "\nset key maxcolumns %d maxrows %d",key->maxcols,key->maxrows);
-    fputc('\n', fp);
-    fprintf(fp, "set key %sopaque\n", key->front ? "" : "no");
-
-    if (!(key->visible))
-	fputs("unset key\n", fp);
+    /* everything about the key */
+    save_key(fp);
 
     fputs("unset label\n", fp);
     for (this_label = first_label; this_label != NULL;
@@ -508,10 +457,11 @@ save_set_all(FILE *fp)
 	    save_linetype(fp, &(this_label->lp_properties), TRUE);
 	}
 	save_position(fp, &this_label->offset, 3, TRUE);
-#ifdef EAM_BOXED_TEXT
-	if (this_label->boxed)
+	if (this_label->boxed) {
 	    fprintf(fp," boxed ");
-#endif
+	    if (this_label->boxed > 0)
+		fprintf(fp,"bs %d ",this_label->boxed);
+	}
 	fputc('\n', fp);
     }
     fputs("unset arrow\n", fp);
@@ -522,7 +472,7 @@ save_set_all(FILE *fp)
 	if (this_arrow->type == arrow_end_absolute) {
 	    fputs(" to ", fp);
 	    save_position(fp, &this_arrow->end, 3, FALSE);
-	} else if (this_arrow->type == arrow_end_absolute) {
+	} else if (this_arrow->type == arrow_end_relative) {
 	    fputs(" rto ", fp);
 	    save_position(fp, &this_arrow->end, 3, FALSE);
 	} else { /* type arrow_end_oriented */
@@ -548,9 +498,10 @@ save_set_all(FILE *fp)
 	}
 	fprintf(fp, "\n");
     }
-#if TRUE || defined(BACKWARDS_COMPATIBLE)
-    fprintf(fp, "set style increment %s\n", prefer_line_styles ? "userstyles" : "default");
-#endif
+
+    /* Mostly for backwards compatibility */
+    if (prefer_line_styles)
+	fprintf(fp, "set style increment userstyles\n");
     fputs("unset style line\n", fp);
     for (this_linestyle = first_linestyle; this_linestyle != NULL;
 	 this_linestyle = this_linestyle->next) {
@@ -558,7 +509,7 @@ save_set_all(FILE *fp)
 	save_linetype(fp, &(this_linestyle->lp_properties), TRUE);
 	fprintf(fp, "\n");
     }
-	/* TODO save "set linetype" as well, or instead */ 
+
     fputs("unset style arrow\n", fp);
     for (this_arrowstyle = first_arrowstyle; this_arrowstyle != NULL;
 	 this_arrowstyle = this_arrowstyle->next) {
@@ -588,19 +539,17 @@ save_set_all(FILE *fp)
     fprintf(fp, "set style histogram ");
     save_histogram_opts(fp);
 
-#ifdef EAM_OBJECTS
+    save_pixmaps(fp);
+
     fprintf(fp, "unset object\n");
     save_object(fp, 0);
-#endif
+    fprintf(fp, "unset walls\n");
+    save_walls(fp);
 
-#ifdef EAM_BOXED_TEXT
-    fprintf(fp, "set style textbox");
     save_style_textbox(fp);
-#endif
 
     save_offsets(fp, "set offsets");
 
-    /* FIXME */
     fprintf(fp, "\
 set pointsize %g\n\
 set pointintervalbox %g\n\
@@ -612,6 +561,12 @@ set encoding %s\n\
 	    (polar) ? "" : "un",
 	    (parametric) ? "" : "un");
 
+    if (spiderplot) {
+	fprintf(fp, "set spiderplot\n");
+	save_style_spider(fp);
+    } else 
+	fprintf(fp, "unset spiderplot\n");
+
     if (numeric_locale)
 	fprintf(fp, "set decimalsign locale \"%s\"\n", numeric_locale);
     if (decimalsign != NULL)
@@ -625,6 +580,10 @@ set encoding %s\n\
     fputs("set view ", fp);
     if (splot_map == TRUE)
 	fprintf(fp, "map scale %g", mapview_scale);
+    else if (xz_projection)
+	fprintf(fp, "projection xz");
+    else if (yz_projection)
+	fprintf(fp, "projection yz");
     else {
 	fprintf(fp, "%g, %g, %g, %g",
 	    surface_rot_x, surface_rot_z, surface_scale, surface_zscale);
@@ -638,12 +597,15 @@ set encoding %s\n\
 
     fprintf(fp, "\n\
 set samples %d, %d\n\
-set isosamples %d, %d\n\
-%sset surface %s",
+set isosamples %d, %d",
 	    samples_1, samples_2,
-	    iso_samples_1, iso_samples_2,
-	    (draw_surface) ? "" : "un",
-	    (implicit_surface) ? "" : "explicit");
+	    iso_samples_1, iso_samples_2);
+
+    fprintf(fp, "\n\
+set surface %s\n\
+%sset surface",
+	    (implicit_surface) ? "implicit" : "explicit",
+	    (draw_surface) ? "" : "un");
 
     fprintf(fp, "\n\
 %sset contour", (draw_contour) ? "" : "un");
@@ -694,6 +656,7 @@ set isosamples %d, %d\n\
 	fprintf(fp, "set datafile fortran\n");
     if (df_nofpe_trap)
 	fprintf(fp, "set datafile nofpe_trap\n");
+    fprintf(fp, "set datafile %scolumnheaders\n", df_columnheaders ? "" : "no");
 
     save_hidden3doptions(fp);
     fprintf(fp, "set cntrparam order %d\n", contour_order);
@@ -709,15 +672,14 @@ set isosamples %d, %d\n\
 	fputs("bspline\n", fp);
 	break;
     }
-    fputs("set cntrparam levels ", fp);
+    fprintf(fp, "set cntrparam levels %d\nset cntrparam levels ", contour_levels);
     switch (contour_levels_kind) {
     case LEVELS_AUTO:
-	fprintf(fp, "auto %d", contour_levels);
+	fprintf(fp, "auto");
 	break;
     case LEVELS_INCREMENTAL:
-	fprintf(fp, "incremental %g,%g,%g",
-		contour_levels_list[0], contour_levels_list[1],
-		contour_levels_list[0] + contour_levels_list[1] * contour_levels);
+	fprintf(fp, "incremental %g,%g",
+		contour_levels_list[0], contour_levels_list[1]);
 	break;
     case LEVELS_DISCRETE:
 	{
@@ -727,8 +689,8 @@ set isosamples %d, %d\n\
 		fprintf(fp, ",%g ", contour_levels_list[i]);
 	}
     }
+    fprintf(fp, "\nset cntrparam firstlinetype %d", contour_firstlinetype);
     fprintf(fp, " %ssorted\n", contour_sortlevels ? "" : "un");
-    fprintf(fp, "set cntrparam firstlinetype %d\n", contour_firstlinetype);
     fprintf(fp, "\
 set cntrparam points %d\n\
 set size ratio %g %g,%g\n\
@@ -778,41 +740,19 @@ set origin %g,%g\n",
     save_tics(fp, &R_AXIS);
     save_tics(fp, &THETA_AXIS);
     for (axis=0; axis<num_parallel_axes; axis++)
-	save_tics(fp, &parallel_axis[axis]);
+	save_tics(fp, &parallel_axis_array[axis]);
 
-#define SAVE_AXISLABEL_OR_TITLE(name,suffix,lab)			 \
-    {									 \
-	fprintf(fp, "set %s%s \"%s\" ",					 \
-		name, suffix, lab.text ? conv_text(lab.text) : "");	 \
-	fprintf(fp, "\nset %s%s ", name, suffix);			 \
-	save_position(fp, &(lab.offset), 3, TRUE);			 \
-	fprintf(fp, " font \"%s\"", lab.font ? conv_text(lab.font) : "");\
-	save_textcolor(fp, &(lab.textcolor));				 \
-	if (lab.pos != CENTRE) save_justification(lab.pos,fp);		 \
-	if (lab.tag == ROTATE_IN_3D_LABEL_TAG)				 \
-	    fprintf(fp, " rotate parallel");				 \
-	else if (lab.rotate == TEXT_VERTICAL)				 \
-	    fprintf(fp, " rotate");					 \
-	else if (lab.rotate)						 \
-	    fprintf(fp, " rotate by %d", lab.rotate);			 \
-	else								 \
-	    fprintf(fp, " norotate");					 \
-	fprintf(fp, "%s\n", (lab.noenhanced) ? " noenhanced" : "");	 \
-    }
+    save_axis_label_or_title(fp, "", "title", &title, TRUE);
 
-    SAVE_AXISLABEL_OR_TITLE("", "title", title);
-
-    /* FIXME */
     fprintf(fp, "set timestamp %s \n", timelabel_bottom ? "bottom" : "top");
-    SAVE_AXISLABEL_OR_TITLE("", "timestamp", timelabel);
+    save_axis_label_or_title(fp, "", "timestamp", &timelabel, FALSE);
 
     save_prange(fp, axis_array + T_AXIS);
     save_prange(fp, axis_array + U_AXIS);
     save_prange(fp, axis_array + V_AXIS);
 
 #define SAVE_AXISLABEL(axis)					\
-    SAVE_AXISLABEL_OR_TITLE(axis_name(axis),"label",	\
-			    axis_array[axis].label)
+    save_axis_label_or_title(fp, axis_name(axis), "label", &axis_array[axis].label, TRUE)
 
     SAVE_AXISLABEL(FIRST_X_AXIS);
     SAVE_AXISLABEL(SECOND_X_AXIS);
@@ -833,11 +773,21 @@ set origin %g,%g\n",
     SAVE_AXISLABEL(POLAR_AXIS);
     save_prange(fp, axis_array + POLAR_AXIS);
 
-    for (axis=0; axis<num_parallel_axes; axis++)
-	save_prange(fp, &parallel_axis[axis]);
+    for (axis=0; axis<num_parallel_axes; axis++) {
+	struct axis *paxis = &parallel_axis_array[axis];
+	save_prange(fp, paxis);
+	if (paxis->label.text)
+	    save_axis_label_or_title(fp,
+				axis_name(paxis->index),"label",
+				&paxis->label, TRUE);
+	if (paxis->zeroaxis) {
+	    fprintf(fp, "set paxis %d", axis+1);
+	    save_linetype(fp, paxis->zeroaxis, FALSE);
+	    fprintf(fp, "\n");
+	}
+    }
 
 #undef SAVE_AXISLABEL
-#undef SAVE_AXISLABEL_OR_TITLE
 
     fputs("unset logscale\n", fp);
     for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++) {
@@ -891,6 +841,8 @@ set origin %g,%g\n",
 	fprintf(fp," noborder");
     } else {
 	fprintf(fp," border");
+	if (pm3d.border.l_type == LT_DEFAULT)
+	    fprintf(fp," retrace");
 	save_linetype(fp, &(pm3d.border), FALSE);
     }
     fputs(" corners2color ", fp);
@@ -908,16 +860,16 @@ set origin %g,%g\n",
     }
     fputs("\n", fp);
 
-#if (0)
     fprintf(fp, "set pm3d %s %s\n",
-		pm3d.clip == PM3D_CLIP_1IN ? "clip1in" : "clip4in",
+		pm3d.clip == PM3D_CLIP_1IN ? "clip1in" : 
+		pm3d.clip == PM3D_CLIP_4IN ? "clip4in" : "clip z",
 		pm3d.no_clipcb ? "noclipcb" : "");
-#endif
 
     if (pm3d_shade.strength <= 0)
 	fputs("set pm3d nolighting\n",fp);
     else
-	fprintf(fp, "set pm3d lighting primary %g specular %g\n", pm3d_shade.strength, pm3d_shade.spec);
+	fprintf(fp, "set pm3d lighting primary %g specular %g spec2 %g\n",
+		pm3d_shade.strength, pm3d_shade.spec, pm3d_shade.spec2);
 
     /*
      *  Save palette information
@@ -1010,13 +962,10 @@ set origin %g,%g\n",
 	fputc('\n', fp);
     }
 
-    fputs("set fontpath ", fp);
-    {
-	char *s;
-	while ((s = save_fontpath()) != NULL)
-	    fprintf(fp, "\"%s\" ", s);
-	fputc('\n', fp);
-    }
+    if (PS_fontpath)
+	fprintf(fp, "set fontpath \"%s\"\n", PS_fontpath);
+    else
+	fprintf(fp, "set fontpath\n");
 
     if (PS_psdir)
 	fprintf(fp, "set psdir \"%s\"\n", PS_psdir);
@@ -1028,20 +977,7 @@ set origin %g,%g\n",
 	fprintf(fp, " nologfile");
     else if (fitlogfile)
 	fprintf(fp, " logfile \'%s\'", fitlogfile);
-    switch (fit_verbosity) {
-	case QUIET:
-	    fprintf(fp, " quiet");
-	    break;
-	case RESULTS:
-	    fprintf(fp, " results");
-	    break;
-	case BRIEF:
-	    fprintf(fp, " brief");
-	    break;
-	case VERBOSE:
-	    fprintf(fp, " verbose");
-	    break;
-    }
+    fprintf(fp, " %s", reverse_table_lookup(fit_verbosity_level, fit_verbosity));
     fprintf(fp, " %serrorvariables",
 	fit_errorvariables ? "" : "no");
     fprintf(fp, " %scovariancevariables",
@@ -1063,7 +999,7 @@ set origin %g,%g\n",
 	    fprintf(fp, " limit_abs %g", epsilon_abs);
 
 	v = get_udv_by_name((char *)FITMAXITER);
-	i = ((v != NULL) && (v->udv_value.type != NOTDEFINED)) ? real_int(&(v->udv_value)) : -1;
+	i = ((v != NULL) && (v->udv_value.type != NOTDEFINED)) ? real(&(v->udv_value)) : -1;
 	if (i > 0)
 	    fprintf(fp, " maxiter %i", i);
 
@@ -1182,6 +1118,120 @@ save_tics(FILE *fp, struct axis *this_axis)
 
 }
 
+static void
+save_key(FILE *fp)
+{
+    legend_key *key = &keyT;
+
+    if (key->title.text == NULL)
+	fprintf(fp, "set key notitle\n");
+    else {
+	fprintf(fp, "set key title \"%s\"", conv_text(key->title.text));
+	if (key->title.font)
+	    fprintf(fp, " font \"%s\" ", key->title.font);
+	save_justification(key->title.pos, fp);
+	fputs("\n", fp);
+    }
+
+    fputs("set key ", fp);
+    switch (key->region) {
+	case GPKEY_AUTO_INTERIOR_LRTBC:
+	    fputs(key->fixed ? "fixed" : "inside", fp);
+	    break;
+	case GPKEY_AUTO_EXTERIOR_LRTBC:
+	    fputs("outside", fp);
+	    break;
+	case GPKEY_AUTO_EXTERIOR_MARGIN:
+	    switch (key->margin) {
+	    case GPKEY_TMARGIN:
+		fputs("tmargin", fp);
+		break;
+	    case GPKEY_BMARGIN:
+		fputs("bmargin", fp);
+		break;
+	    case GPKEY_LMARGIN:
+		fputs("lmargin", fp);
+		break;
+	    case GPKEY_RMARGIN:
+		fputs("rmargin", fp);
+		break;
+	    }
+	    break;
+	case GPKEY_USER_PLACEMENT:
+	    fputs("at ", fp);
+	    save_position(fp, &key->user_pos, 2, FALSE);
+	    break;
+    }
+    if (!(key->region == GPKEY_AUTO_EXTERIOR_MARGIN
+	      && (key->margin == GPKEY_LMARGIN || key->margin == GPKEY_RMARGIN))) {
+	save_justification(key->hpos, fp);
+    }
+    if (!(key->region == GPKEY_AUTO_EXTERIOR_MARGIN
+	      && (key->margin == GPKEY_TMARGIN || key->margin == GPKEY_BMARGIN))) {
+	switch (key->vpos) {
+	    case JUST_TOP:
+		fputs(" top", fp);
+		break;
+	    case JUST_BOT:
+		fputs(" bottom", fp);
+		break;
+	    case JUST_CENTRE:
+		fputs(" center", fp);
+		break;
+	}
+    }
+    fprintf(fp, " %s %s %sreverse %senhanced %s ",
+		key->stack_dir == GPKEY_VERTICAL ? "vertical" : "horizontal",
+		key->just == GPKEY_LEFT ? "Left" : "Right",
+		key->reverse ? "" : "no",
+		key->enhanced ? "" : "no",
+		key->auto_titles == COLUMNHEAD_KEYTITLES ? "autotitle columnhead"
+		: key->auto_titles == FILENAME_KEYTITLES ? "autotitle"
+		: "noautotitle" );
+    if (key->box.l_type > LT_NODRAW) {
+	fputs("box", fp);
+	save_linetype(fp, &(key->box), FALSE);
+    } else
+	fputs("nobox", fp);
+
+    /* These are for the key entries, not the key title */
+    if (key->font)
+	fprintf(fp, " font \"%s\"", key->font);
+    if (key->textcolor.type != TC_LT || key->textcolor.lt != LT_BLACK)
+	save_textcolor(fp, &key->textcolor);
+
+    /* Put less common options on separate lines */
+    fprintf(fp, "\nset key %sinvert samplen %g spacing %g width %g height %g\n",
+		key->invert ? "" : "no",
+		key->swidth, key->vert_factor, key->width_fix, key->height_fix);
+    if (key->user_width.x > 0 || key->user_cols > 0) {
+	fprintf(fp, "set key ");
+	if (key->user_cols > 0)
+	    fprintf(fp, "columns %d ", key->user_cols);
+	if (key->user_width.x > 0) {
+	    fprintf(fp, "keywidth ");
+	    save_position(fp, &key->user_width, 1, FALSE);
+	}
+	fprintf(fp, "\n");
+    }
+    fprintf(fp, "set key maxcolumns %d maxrows %d",key->maxcols,key->maxrows);
+    save_position(fp, &key->offset, 2, TRUE);
+    fprintf(fp, "\n");
+    if (key->front) {
+	fprintf(fp, "set key opaque");
+	if (key->fillcolor.lt != LT_BACKGROUND) {
+	    fprintf(fp, " fc ");
+	    save_pm3dcolor(fp, &key->fillcolor);
+	}
+	fprintf(fp, "\n");
+    } else {
+	fprintf(fp, "set key noopaque\n");
+    }
+
+    if (!(key->visible))
+	fputs("unset key\n", fp);
+}
+
 static void
 save_mtics(FILE *fp, struct axis *axis)
 {
@@ -1240,27 +1290,43 @@ save_style_parallel(FILE *fp)
     fprintf(fp, "\n");
 }
 
-#ifdef EAM_BOXED_TEXT
+void
+save_style_spider(FILE *fp)
+{
+    fprintf(fp, "set style spiderplot ");
+    save_linetype(fp, &(spiderplot_style.lp_properties), TRUE);
+    fprintf(fp, "\nset style spiderplot fillstyle ");
+    save_fillstyle(fp, &spiderplot_style.fillstyle);
+}
+
 void
 save_style_textbox(FILE *fp)
 {
-    fprintf(fp, " %s margins %4.1f, %4.1f",
-	    textbox_opts.opaque ? "opaque": "transparent",
-	    textbox_opts.xmargin, textbox_opts.ymargin);
-    if (textbox_opts.opaque) {
-	fprintf(fp, " fc ");
-	save_pm3dcolor(fp, &(textbox_opts.fillcolor));
+    int bs;
+    for (bs = 0; bs < NUM_TEXTBOX_STYLES; bs++) {
+	textbox_style *textbox = &textbox_opts[bs];
+	if (textbox->linewidth <= 0)
+	    continue;
+	fprintf(fp, "set style textbox ");
+	if (bs > 0)
+	    fprintf(fp,"%d ", bs);
+	fprintf(fp, " %s margins %4.1f, %4.1f",
+		textbox->opaque ? "opaque": "transparent",
+		textbox->xmargin, textbox->ymargin);
+	if (textbox->opaque) {
+	    fprintf(fp, " fc ");
+	    save_pm3dcolor(fp, &(textbox->fillcolor));
+	}
+	if (textbox->noborder) {
+	    fprintf(fp, " noborder");
+	} else {
+	    fprintf(fp, " border ");
+	    save_pm3dcolor(fp, &(textbox->border_color));
+	}
+	fprintf(fp, " linewidth %4.1f", textbox->linewidth);
+	fputs("\n",fp);
     }
-    if (textbox_opts.noborder) {
-	fprintf(fp, " noborder");
-    } else {
-	fprintf(fp, " border ");
-	save_pm3dcolor(fp, &(textbox_opts.border_color));
-    }
-    fprintf(fp, " linewidth %4.1f", textbox_opts.linewidth);
-    fputs("\n",fp);
 }
-#endif
 
 void
 save_position(FILE *fp, struct position *pos, int ndim, TBOOLEAN offset)
@@ -1309,6 +1375,8 @@ save_position(FILE *fp, struct position *pos, int ndim, TBOOLEAN offset)
 void
 save_prange(FILE *fp, struct axis *this_axis)
 {
+    TBOOLEAN noextend = FALSE;
+
     fprintf(fp, "set %srange [ ", axis_name(this_axis->index));
     if (this_axis->set_autoscale & AUTOSCALE_MIN) {
 	if (this_axis->min_constraint & CONSTRAINT_LOWER ) {
@@ -1338,30 +1406,30 @@ save_prange(FILE *fp, struct axis *this_axis)
 	save_num_or_time_input(fp, this_axis->set_max, this_axis);
     }
 
-    fprintf(fp, " ] %sreverse %swriteback",
+    if (this_axis->index < PARALLEL_AXES)
+	fprintf(fp, " ] %sreverse %swriteback",
 	    ((this_axis->range_flags & RANGE_IS_REVERSED)) ? "" : "no",
 	    this_axis->range_flags & RANGE_WRITEBACK ? "" : "no");
+    else
+	fprintf(fp, " ] ");
 
-    if (this_axis->index >= PARALLEL_AXES) {
-	fprintf(fp, "\n");
-	return;
+    if ((this_axis->set_autoscale & AUTOSCALE_FIXMIN)
+    &&  (this_axis->set_autoscale & AUTOSCALE_FIXMAX)) {
+	fprintf(fp, " noextend");
+	noextend = TRUE;
     }
 
     if (this_axis->set_autoscale && fp == stderr) {
 	/* add current (hidden) range as comments */
 	fputs("  # (currently [", fp);
-	if (this_axis->set_autoscale & AUTOSCALE_MIN) {
-	    save_num_or_time_input(fp, this_axis->min, this_axis);
-	}
+	save_num_or_time_input(fp, this_axis->min, this_axis);
 	putc(':', fp);
-	if (this_axis->set_autoscale & AUTOSCALE_MAX) {
-	    save_num_or_time_input(fp, this_axis->max, this_axis);
-	}
+	save_num_or_time_input(fp, this_axis->max, this_axis);
 	fputs("] )\n", fp);
     } else
 	putc('\n', fp);
 
-    if (fp != stderr) {
+    if (!noextend && (fp != stderr)) {
 	if (this_axis->set_autoscale & (AUTOSCALE_FIXMIN))
 	    fprintf(fp, "set autoscale %sfixmin\n", axis_name(this_axis->index));
 	if (this_axis->set_autoscale & AUTOSCALE_FIXMAX)
@@ -1386,7 +1454,6 @@ save_link(FILE *fp, AXIS *this_axis)
 void
 save_nonlinear(FILE *fp, AXIS *this_axis)
 {
-#ifdef NONLINEAR_AXES
     AXIS *primary = this_axis->linked_to_primary;
 
     if (primary &&  this_axis->index == -primary->index) {
@@ -1401,7 +1468,6 @@ save_nonlinear(FILE *fp, AXIS *this_axis)
 	    fprintf(stderr, "[corrupt linkage] ");
 	fputs("\n", fp);
     }
-#endif
 }
 
 static void
@@ -1500,111 +1566,26 @@ save_pm3dcolor(FILE *fp, const struct t_colorspec *tc)
 void
 save_data_func_style(FILE *fp, const char *which, enum PLOT_STYLE style)
 {
-    switch (style) {
-    case LINES:
-	fputs("lines\n", fp);
-	break;
-    case POINTSTYLE:
-	fputs("points\n", fp);
-	break;
-    case IMPULSES:
-	fputs("impulses\n", fp);
-	break;
-    case LINESPOINTS:
-	fputs("linespoints\n", fp);
-	break;
-    case DOTS:
-	fputs("dots\n", fp);
-	break;
-    case YERRORLINES:
-	fputs("yerrorlines\n", fp);
-	break;
-    case XERRORLINES:
-	fputs("xerrorlines\n", fp);
-	break;
-    case XYERRORLINES:
-	fputs("xyerrorlines\n", fp);
-	break;
-    case YERRORBARS:
-	fputs("yerrorbars\n", fp);
-	break;
-    case XERRORBARS:
-	fputs("xerrorbars\n", fp);
-	break;
-    case XYERRORBARS:
-	fputs("xyerrorbars\n", fp);
-	break;
-    case BOXES:
-	fputs("boxes\n", fp);
-	break;
-    case HISTOGRAMS:
-	fputs("histograms\n", fp);
-	break;
-    case FILLEDCURVES:
-	fputs("filledcurves ", fp);
+    char *answer = strdup(reverse_table_lookup(plotstyle_tbl, style));
+    char *idollar = strchr(answer, '$');
+    if (idollar) {
+	do {
+	    *idollar = *(idollar+1);
+	    idollar++;
+	} while (*idollar);
+
+    }
+    fputs(answer, fp);
+    free(answer);
+
+    if (style == FILLEDCURVES) {
+	fputs(" ", fp);
 	if (!strcmp(which, "data") || !strcmp(which, "Data"))
 	    filledcurves_options_tofile(&filledcurves_opts_data, fp);
 	else
 	    filledcurves_options_tofile(&filledcurves_opts_func, fp);
-	fputc('\n', fp);
-	break;
-    case BOXERROR:
-	fputs("boxerrorbars\n", fp);
-	break;
-    case BOXXYERROR:
-	fputs("boxxyerror\n", fp);
-	break;
-    case STEPS:
-	fputs("steps\n", fp);
-	break;			/* JG */
-    case FSTEPS:
-	fputs("fsteps\n", fp);
-	break;			/* HOE */
-    case HISTEPS:
-	fputs("histeps\n", fp);
-	break;			/* CAC */
-    case VECTOR:
-	fputs("vector\n", fp);
-	break;
-    case FINANCEBARS:
-	fputs("financebars\n", fp);
-	break;
-    case CANDLESTICKS:
-	fputs("candlesticks\n", fp);
-	break;
-    case BOXPLOT:
-	fputs("boxplot\n", fp);
-	break;
-    case PM3DSURFACE:
-	fputs("pm3d\n", fp);
-	break;
-    case LABELPOINTS:
-	fputs("labels\n", fp);
-	break;
-    case IMAGE:
-	fputs("image\n", fp);
-	break;
-    case RGBIMAGE:
-	fputs("rgbimage\n", fp);
-	break;
-#ifdef EAM_OBJECTS
-	case CIRCLES:
-	fputs("circles\n", fp);
-	break;
-	case ELLIPSES:
-	fputs("ellipses\n", fp);
-	break;
-#endif
-    case SURFACEGRID:
-	fputs("surfaces\n", fp);
-	break;
-    case PARALLELPLOT:
-	fputs("parallelaxes\n", fp);
-	break;
-    case PLOT_STYLE_NONE:
-    default:
-	fputs("---error!---\n", fp);
     }
+    fputc('\n', fp);
 }
 
 void
@@ -1729,10 +1710,27 @@ save_histogram_opts (FILE *fp)
     if (histogram_opts.title.font)
 	fprintf(fp, " font \"%s\" ", histogram_opts.title.font);
     save_position(fp, &histogram_opts.title.offset, 2, TRUE);
+    if (!histogram_opts.keyentry)
+	fprintf(fp, " nokeyseparators");
     fprintf(fp, "\n");
 }
 
-#ifdef EAM_OBJECTS
+void
+save_pixmaps(FILE *fp)
+{
+    t_pixmap *pixmap;
+    for (pixmap = pixmap_listhead; pixmap; pixmap = pixmap->next) {
+	fprintf(fp, "set pixmap %d '%s' # (%d x %d pixmap)\n",
+		pixmap->tag, pixmap->filename, pixmap->ncols, pixmap->nrows);
+	fprintf(fp, "set pixmap %d at ", pixmap->tag);
+	save_position(fp,&pixmap->pin, 3, FALSE);
+	fprintf(fp, "  size ");
+	save_position(fp,&pixmap->extent, 2, FALSE);
+	fprintf(fp, " %s %s\n",
+		pixmap->layer == LAYER_FRONT ? "front" : "behind",
+		pixmap->center ? "center" : "");
+    }
+}
 
 /* Save/show rectangle <tag> (0 means show all) */
 void
@@ -1824,7 +1822,10 @@ save_object(FILE *fp, int tag)
 	/* Properties common to all objects */
 	if (tag == 0 || tag == this_object->tag) {
 	    fprintf(fp, "\n%sobject %2d ", (fp==stderr) ? "\t" : "set ",this_object->tag);
-	    fprintf(fp, "%s ", this_object->layer > 0 ? "front" : this_object->layer < 0 ? "behind" : "back");
+	    fprintf(fp, "%s ", this_object->layer == LAYER_FRONT ? "front" :
+				this_object->layer == LAYER_DEPTHORDER ? "depthorder" :
+				this_object->layer == LAYER_BEHIND ? "behind" :
+				"back");
 	    if (this_object->clip == OBJ_NOCLIP)
 		fputs("noclip ", fp);
 	    else 
@@ -1838,7 +1839,7 @@ save_object(FILE *fp, int tag)
 	    fprintf(fp, " fc ");
 	    if (this_object->lp_properties.l_type == LT_DEFAULT)
 		    fprintf(fp,"default");
-	    else /* FIXME: Broke with removal of use_palette? */
+	    else
 		    save_pm3dcolor(fp, &this_object->lp_properties.pm3d_color);
 	    fprintf(fp, " fillstyle ");
 	    save_fillstyle(fp, &this_object->fillstyle);
@@ -1849,5 +1850,24 @@ save_object(FILE *fp, int tag)
 	int_error(c_token, "object not found");
 }
 
-#endif
+/* Save/show special polygon objects created by "set wall" */
+void
+save_walls(FILE *fp)
+{
+    static const char* wall_name[5] = {"y0", "x0", "y1", "x1", "z0"};
+    t_object *this_object;
+    int i;
+
+    for (i = 0; i < 5; i++) {
+    	this_object = &grid_wall[i];
+	if (this_object->layer == LAYER_FRONTBACK) {
+	    fprintf(fp, "set wall %s ", wall_name[i]);
+	    fprintf(fp, " fc ");
+	    save_pm3dcolor(fp, &this_object->lp_properties.pm3d_color);
+	    fprintf(fp, " fillstyle ");
+	    save_fillstyle(fp, &this_object->fillstyle);
+	}
+
+    }
+}
 
