@@ -1,7 +1,3 @@
-/*
- * $Id: stdfn.h,v 1.40 2011/09/21 11:43:50 markisch Exp $
- */
-
 /* GNUPLOT - stdfn.h */
 
 /*[
@@ -64,7 +60,7 @@
 # endif
 #else
 # ifndef HAVE_MEMCPY
-char * memcpy __PROTO((char *, char *, size_t));
+char * memcpy(char *, char *, size_t);
 # endif
 #endif /* HAVE_BCOPY */
 
@@ -75,7 +71,7 @@ char * memcpy __PROTO((char *, char *, size_t));
 # ifdef HAVE_INDEX
 #  define strchr index
 # else
-char *strchr __PROTO((const char *, int));
+char *strchr(const char *, int);
 # endif
 # ifdef strrchr
 #  undef strrchr
@@ -85,12 +81,12 @@ char *strchr __PROTO((const char *, int));
 # endif
 #endif
 #ifndef HAVE_STRCSPN
-size_t gp_strcspn __PROTO((const char *, const char *));
+size_t gp_strcspn(const char *, const char *);
 # define strcspn gp_strcspn
 #endif
 
 #ifndef HAVE_STRSTR
-char *strstr __PROTO((const char *, const char *));
+char *strstr(const char *, const char *);
 #endif
 
 #ifndef HAVE_STDLIB_H
@@ -150,7 +146,7 @@ double strtod();
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #else
-# ifdef HAVE_LIBC_H /* NeXT uses libc instead of unistd */
+# ifdef HAVE_LIBC_H
 #  include <libc.h>
 # endif
 #endif /* HAVE_UNISTD_H */
@@ -162,7 +158,7 @@ double strtod();
 extern int errno;
 #endif
 #ifndef HAVE_STRERROR
-char *strerror __PROTO((int));
+char *strerror(int);
 extern int sys_nerr;
 extern char *sys_errlist[];
 #endif
@@ -266,8 +262,8 @@ extern char *sys_errlist[];
 #endif
 
 #if defined(PIPES) && defined(VMS)
-FILE *popen __PROTO((char *, char *));
-int pclose __PROTO((FILE *));
+FILE *popen(char *, char *);
+int pclose(FILE *);
 #endif
 
 #ifdef HAVE_FLOAT_H
@@ -310,7 +306,7 @@ int pclose __PROTO((FILE *));
 # ifdef HAVE_STRICMP
 #  define strcasecmp stricmp
 # else
-int gp_stricmp __PROTO((const char *, const char *));
+int gp_stricmp(const char *, const char *);
 #  define strcasecmp gp_stricmp
 # endif
 #endif
@@ -319,21 +315,29 @@ int gp_stricmp __PROTO((const char *, const char *));
 # ifdef HAVE_STRNICMP
 #  define strncasecmp strnicmp
 # else
-int gp_strnicmp __PROTO((const char *, const char *, size_t));
+int gp_strnicmp(const char *, const char *, size_t);
 #  define strncasecmp gp_strnicmp
 # endif
 #endif
 
 #ifndef HAVE_STRNDUP
-char *strndup __PROTO((const char * str, size_t n));
+char *strndup(const char * str, size_t n);
 #endif
 
 #ifndef HAVE_STRNLEN
-size_t strnlen __PROTO((const char *str, size_t n));
+size_t strnlen(const char *str, size_t n);
+#endif
+
+#if defined(_MSC_VER) && (_MSC_VER < 1900)
+int ms_vsnprintf(char *str, size_t size, const char *format, va_list ap);
+int ms_snprintf(char *str, size_t size, const char * format, ...);
 #endif
 
 #ifndef GP_GETCWD
-# if defined(HAVE_GETCWD)
+# if defined(_WIN32)
+#  define GP_GETCWD(path,len) gp_getcwd (path, len)
+char * gp_getcwd(char *path, size_t len);
+# elif defined(HAVE_GETCWD)
 #   if defined(__EMX__)
 #     define GP_GETCWD(path,len) _getcwd2 (path, len)
 #   else
@@ -344,12 +348,8 @@ size_t strnlen __PROTO((const char *str, size_t n));
 # endif
 #endif
 
-#ifdef WIN32
-# include <windows.h>
-#endif
-
 /* sleep delay time, where delay is a double value */
-#if defined(HAVE_USLEEP)
+#if defined(HAVE_USLEEP) && !defined(_WIN32)
 #  define GP_SLEEP(delay) usleep((unsigned int) ((delay)*1e6))
 #  ifndef HAVE_SLEEP
 #    define HAVE_SLEEP
@@ -359,7 +359,7 @@ size_t strnlen __PROTO((const char *str, size_t n));
 #  ifndef HAVE_SLEEP
 #    define HAVE_SLEEP
 #  endif
-#elif defined(WIN32)
+#elif defined(_WIN32)
 #  define GP_SLEEP(delay) win_sleep((DWORD) 1000*delay)
 #  ifndef HAVE_SLEEP
 #    define HAVE_SLEEP
@@ -370,17 +370,25 @@ size_t strnlen __PROTO((const char *str, size_t n));
 #  define GP_SLEEP(delay) sleep ((unsigned int) (delay+0.5))
 #endif
 
-#ifdef HAVE_ATEXIT
-# define GP_ATEXIT(x) atexit((x))
-#elif defined(HAVE_ON_EXIT)
-# define GP_ATEXIT(x) on_exit((x),0)
-#else
-# define GP_ATEXIT(x) /* you lose */
-#endif
+/* Gnuplot replacement for atexit(3) */
+void gp_atexit(void (*function)(void));
 
-char * gp_basename __PROTO((char *path));
+/* Gnuplot replacement for exit(3). Calls the functions registered using
+ * gp_atexit(). Always use this function instead of exit(3)!
+ */
+void gp_exit(int status);
 
-#if !defined(HAVE_DIRENT_H) && defined(WIN32) && (!defined(__WATCOMC__))
+/* Calls the cleanup functions registered using gp_atexit().
+ * Normally gnuplot should be exited using gp_exit(). In some cases, this is not
+ * possible (notably when returning from main(), where some compilers get
+ * confused because they expect a return statement at the very end. In that
+ * case, gp_exit_cleanup() should be called before the return statement.
+ */
+void gp_exit_cleanup(void);
+
+char * gp_basename(char *path);
+
+#ifdef _WIN32
 /*
 
     Declaration of POSIX directory browsing functions and types for Win32.
@@ -396,28 +404,39 @@ char * gp_basename __PROTO((char *path));
     documentation for any purpose is hereby granted without fee, provided
     that this copyright and permissions notice appear in all copies and
     derivatives.
-    
+
     This software is supplied "as is" without express or implied warranty.
 
     But that said, if there are any problems please get in touch.
 
 */
-typedef struct DIR DIR;
+typedef struct GPDIR GPDIR;
 
-struct dirent
+struct gp_dirent
 {
     char *d_name;
 };
 
-DIR           *opendir __PROTO((const char *));
-int           closedir __PROTO((DIR *));
-struct dirent *readdir __PROTO((DIR *));
-void          rewinddir __PROTO((DIR *));
+GPDIR * gp_opendir(const char *);
+int gp_closedir(GPDIR *);
+struct gp_dirent *gp_readdir(GPDIR *);
+void gp_rewinddir(GPDIR *);
+#define opendir(p) gp_opendir(p)
+#define closedir(d) gp_closedir(d)
+#define readdir(d) gp_readdir(d)
+#define rewinddir(d) gp_rewinddir(d)
+#define dirent gp_dirent
+#define DIR GPDIR
 #elif defined(HAVE_DIRENT_H)
 # include <sys/types.h>
 # include <dirent.h>
-#endif /* !HAVE_DIRENT_H && WIN32 */
+#elif defined(__WATCOMC__)
+# include <direct.h>
+#endif /* !HAVE_DIRENT_H && _WIN32 */
 
+#if defined(HAVE_DIRENT_H) || defined(_WIN32) || defined(__WATCOMC__)
+# define HAVE_DIRENT
+#endif
 
 /* Misc. defines */
 
@@ -436,41 +455,14 @@ void          rewinddir __PROTO((DIR *));
 # define DEBUG_WHERE do { fprintf(stderr,"%s:%d ",__FILE__,__LINE__); } while (0)
 # define FPRINTF(a) do { DEBUG_WHERE; fprintf a; } while (0)
 #else
-# define DEBUG_WHERE     /* nought */
-# define FPRINTF(a)      /* nought */
+# define DEBUG_WHERE     do { /* nought */ } while(0)
+# define FPRINTF(a)      do { /* nought */ } while(0)
 #endif /* DEBUG */
 
 #include "syscfg.h"
 
 #define INT_STR_LEN (3*sizeof(int))
 
-
-/* HBB 20010223: moved this whole block from syscfg.h to here. It
- * needs both "syscfg.h" and <float.h> to have been #include'd before
- * this, since it relies on stuff like DBL_MAX */
-
-/* There is a bug in the NEXT OS. This is a workaround. Lookout for
- * an OS correction to cancel the following dinosaur
- *
- * Hm, at least with my setup (compiler version 3.1, system 3.3p1),
- * DBL_MAX is defined correctly and HUGE and HUGE_VAL are both defined
- * as 1e999. I have no idea to which OS version the bugfix below
- * applies, at least wrt. HUGE, it is inconsistent with the current
- * version. Since we are using DBL_MAX anyway, most of this isn't
- * really needed anymore.
- */
-
-#if defined ( NEXT ) && NX_CURRENT_COMPILER_RELEASE<310
-# if defined ( DBL_MAX)
-#  undef DBL_MAX
-# endif
-# define DBL_MAX 1.7976931348623157e+308
-# undef HUGE
-# define HUGE    DBL_MAX
-# undef HUGE_VAL
-# define HUGE_VAL DBL_MAX
-#endif /* NEXT && NX_CURRENT_COMPILER_RELEASE<310 */
-
 /*
  * Note about VERYLARGE:  This is the upper bound double (or float, if PC)
  * numbers. This flag indicates very large numbers. It doesn't have to
@@ -522,8 +514,12 @@ void          rewinddir __PROTO((DIR *));
 #ifdef HAVE_SYS_PARAM_H
 # include <sys/param.h>
 #endif
+/* HBB 2012-03-18: clang brings its own <limits.h>, which lacks PATH_MAX,
+ * and on top of that, Cygwin's MAXPATHLEN is defined by reference to PATH_MAX.
+ * So even though clang sees a #defined MAXPATHLEN, there's still no
+ * definition available OA*/
 #ifndef PATH_MAX
-# ifndef MAXPATHLEN
+# if !defined(MAXPATHLEN) || (MAXPATHLEN <= 0)
 #  define PATH_MAX 1024
 # else
 #  define PATH_MAX MAXPATHLEN
@@ -534,15 +530,19 @@ void          rewinddir __PROTO((DIR *));
  * may or may not end with a "directory separation" character.
  * Path must not be NULL, but can be empty
  */
+#ifndef VMS
 #define PATH_CONCAT(path,file) \
  { char *p = path; \
    p += strlen(path); \
    if (p!=path) p--; \
    if (*p && (*p != DIRSEP1) && (*p != DIRSEP2)) { \
-     if (*p) p++; *p++ = DIRSEP1; *p = NUL; \
+     if (*p) {p++;} *p++ = DIRSEP1; *p = NUL; \
    } \
    strcat (path, file); \
  }
+#else
+#define PATH_CONCAT(path,file) strcat(path,file)
+#endif
 
 #ifndef inrange
 # define inrange(z,min,max) \
@@ -551,7 +551,7 @@ void          rewinddir __PROTO((DIR *));
 #endif
 
 /* HBB 20030117: new macro to simplify clipping operations in the
- * presence of possibly reverted axes */
+ * presence of possibly reversed axes */
 #ifndef cliptorange
 # define cliptorange(z,min,max)			\
     do {					\
@@ -569,21 +569,29 @@ void          rewinddir __PROTO((DIR *));
     } while (0)
 #endif
 
+#ifndef clip_to_01
+#define clip_to_01(val)	\
+    ((val) < 0 ? 0 : (val) > 1 ? 1 : (val))
+#endif
+
 /* both min/max and MIN/MAX are defined by some compilers.
  * we are now on GPMIN / GPMAX
  */
 #define GPMAX(a,b) ( (a) > (b) ? (a) : (b) )
 #define GPMIN(a,b) ( (a) < (b) ? (a) : (b) )
 
+/*
+ * Do any supported platforms already have a sgn function?
+ */
+#define sgn(x) (((x) > 0) ? 1 : (((x) < 0) ? -1 : 0))
+
 /* Prototypes from "stdfn.c" */
 
-char *safe_strncpy __PROTO((char *, const char *, size_t));
+char *safe_strncpy(char *, const char *, size_t);
 #ifndef HAVE_SLEEP
-unsigned int sleep __PROTO((unsigned int));
+unsigned int sleep(unsigned int);
 #endif
 
-double gp_strtod __PROTO((const char *str, char **endptr));
-
-double not_a_number __PROTO((void));
+double not_a_number(void);
 
 #endif /* STDFN_H */
