@@ -394,6 +394,7 @@ const struct gen_table set_key_tbl[] =
     { "def$ault", S_KEY_DEFAULT },
     { "on", S_KEY_ON },
     { "off", S_KEY_OFF },
+    { "offset", S_KEY_OFFSET},
     { "t$op", S_KEY_TOP },
     { "b$ottom", S_KEY_BOTTOM },
     { "l$eft", S_KEY_LEFT },
@@ -427,6 +428,7 @@ const struct gen_table set_key_tbl[] =
     { "sp$acing", S_KEY_SPACING },
     { "w$idth", S_KEY_WIDTH },
     { "h$eight", S_KEY_HEIGHT },
+    { "keyw$idth", S_KEY_KEYWIDTH },
     { "a$utotitles", S_KEY_AUTOTITLES },
     { "noa$utotitles", S_KEY_NOAUTOTITLES },
     { "ti$tle", S_KEY_TITLE },
@@ -434,6 +436,8 @@ const struct gen_table set_key_tbl[] =
     { "font", S_KEY_FONT },
     { "tc", S_KEY_TEXTCOLOR },
     { "text$color", S_KEY_TEXTCOLOR },
+    { "col$s", S_KEY_COLS},
+    { "colu$mns", S_KEY_COLS},
     { "maxcol$s", S_KEY_MAXCOLS},
     { "maxcolu$mns", S_KEY_MAXCOLS},
     { "maxrow$s", S_KEY_MAXROWS},
